/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX/HB9DRD
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Moon.h"
#include "Inline.h"

#include <cmath>
using namespace std;

const double R  = 6378.16;
const double C1 = 0.9174640600000014;
const double S1 = 0.397818675;

CMoon::CMoon() :
m_az(0.0),
m_el(0.0),
m_ra(0.0),
m_dec(0.0),
m_lha(0.0),
m_lat(53.50),
m_lon(-1.55),
m_ro(0.0),
m_mm(0.0),
m_range(0.0),
m_frequency(1296.0)
{
}

CMoon::~CMoon()
{
}

void CMoon::findMoon(double dayNum)
{
	double gm = 24.0 * (dayNum - ::floor(dayNum));
	double ew = fixAngle(1.134193 + dayNum * 0.2299715060000012);

	m_mm = fixAngle(1.319238 + dayNum * 0.228027135);

	double t1 = fixAngle(6.217512000000011 + dayNum * 0.01720196977);
	double t2 = 2.0 * fixAngle(2.550677 + dayNum * 0.212768711);
	double t3 = fixAngle(4.7652214 + dayNum * 0.230895723);

	ew += 0.01148 * ::sin(t2) + 0.10976 * ::sin(m_mm);
	ew -= 0.022235 * ::sin(m_mm - t2) + 0.003246 * ::sin(t1);
	ew += 0.003735 * ::sin(2.0 * m_mm) - 0.0019897 * ::sin(2.0 * t3);
	ew -= 0.0010297 * ::sin(2.0 * m_mm - t2) + 0.0009948 * ::sin(m_mm + t1 - t2);

	double en = t3 + 0.0115070 * ::sin(t2) + 0.10873924 * ::sin(m_mm);
	en -= 0.0222006 * ::sin(m_mm - t2);
	en = 0.0897797 * ::sin(en) - 0.002548 * ::sin(t3 - t2);

	double si = C1 * ::sin(en) + S1 * ::cos(en) * ::sin(ew);
	double co = ::sqrt(1.0 - (si * si));
	m_dec = ::atan2(si, co);

	si = ::sin(ew) * C1 - ::tan(en) * S1;
	co = ::cos(ew);
	m_ra = ::atan2(si, co);

	if (m_ra < 0.0)
		m_ra += 2.0 * M_PI;

	t1 = double(long((dayNum - 39410.0)));
	t2 = ::floor((t1 + 32044.75) / 36524.25);
	t2 += t1 - t2 / 4.0 + 1486.0;
	double yr = 2084.0 + ::floor((t2 - 122.1) / 365.25);

	t1 = yr - 1.0;
	double de = ::floor(365.25 * (t1 - 1980.0)) - ::floor(t1 / 100.0) + ::floor(t1 / 400.0) + 381.0;

	t1 = (de + 29218.5) / 36525.0;
	t1 = 6.6460656 + t1 * (2400.051262 + t1 * 2.581e-5);
	double se = t1 - 24.0 * (yr - 1900.0);

	t1 = (se + 0.0657098 * (dayNum - de) + gm * 1.002737909350795) / 24.0;
	double gs = 24.0 * (t1 - ::floor(t1));

	t1 = gs / 24.0 - m_ra / (2.0 * M_PI);
	double gh = 2.0 * M_PI * (t1 - ::floor(t1));

	m_lha = gh + m_lon;

	si = ::cos(m_lha) * ::cos(m_dec) * ::cos(m_lat) + ::sin(m_dec) * ::sin(m_lat);
	co = ::sqrt(1.0 - (si * si));
	m_el = ::atan2(si, co);

	si = -::sin(m_lha) * ::cos(m_dec) * ::cos(m_lat);
	co = ::sin(m_dec) - ::sin(m_lat) * ::sin(m_el);
	m_az = ::atan2(si, co);

	if (m_az < 0.0)
		m_az += 2.0 * M_PI;

	m_ro = 0.996986 / (1.0 + 0.0549 * ::cos(m_mm + 0.10976 * ::sin(m_mm)));
	m_el -= 0.0166 * ::cos(m_el) / m_ro;

	t1 = R * 1000.0;
	t2 = 384401.0 * m_ro * 1000.0;
	m_range = ::sqrt(t2 * t2 + 2.0 * t1 * t2 * ::sin(m_el));
}

double CMoon::getAzimuth() const
{
	return DEG(m_az);
}

double CMoon::getElevation() const
{
	return DEG(m_el);
}

double CMoon::getRA() const
{
	return DEG(m_ra);
}

double CMoon::getDec() const
{
	return DEG(m_dec);
}

double CMoon::getLHA() const
{
	return DEG(m_lha);
}

double CMoon::getDoppler() const
{
	double t2 = 0.10976;
	double t1 = m_mm + t2 + ::sin(m_mm);
	double dmdt = 0.01255 * m_ro * m_ro * ::sin(t1) * (1.0 + t2 * ::cos(m_mm));
	dmdt *= 4449.0;

	t1 = R * 1000.0;
	t2 = 384401.0 * m_ro * 1000.0;

	double drdt1 = dmdt * (t2 + (t1 * ::sin(m_el))) / m_range;
	double dlhdt = 7.53125e-5 * (::sin(m_lha) * ::cos(m_dec) * ::cos(m_lat));

	double    dcMax = RAD(20.0);
	double ddcdtMax = RAD(0.0000716);

	double t3 = m_dec / dcMax;
	if (t3 < 1.0) {
		t3 = ::sin(t3);
		double ddcdt = ddcdtMax * ::sqrt(1.0 - t3 * t3);
		t3 = ddcdt * (::cos(m_dec) * ::sin(m_lat) - ::cos(m_lha) * ::sin(m_dec) * ::cos(m_lat));
	} else {
		t3 = 0.0;
	}

	double dsineldt = dlhdt + t3;
	double    drdt2 = t1 * t2 * dsineldt / m_range;

	double dv = drdt1 + drdt2;

	return -dv * m_frequency / 299.810;
}

double CMoon::getPathLoss() const
{
	return -17.37 * ::log10(m_ro);
}

double CMoon::getRange() const
{
	return m_range;
}

void CMoon::setLocation(double latitude, double longitude)
{
	m_lat = RAD(latitude);
	m_lon = RAD(longitude);
}

void CMoon::setFrequency(double frequency)
{
	m_frequency = frequency;
}

double CMoon::fixAngle(double angle) const
{
	/* This function reduces angles greater than
	   two pi by subtracting two pi from the angle */

	while (angle > (2.0 * M_PI))
		angle -= 2.0 * M_PI;

	return angle;
}

double CMoon::getDayNum(const wxDateTime& dateTime) const
{
	int  secs = dateTime.GetTicks();
	int msecs = dateTime.GetMillisecond();

	return ((double(secs) + 0.001 * double(msecs)) / 86400.0) - 3651.0;
}
