/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX/HB9DRD
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "MoonPanel.h"

#include "common/Locator.h"

#include <cmath>

BEGIN_EVENT_TABLE(CMoonPanel, wxPanel)
	EVT_PAINT(CMoonPanel::onPaint)
END_EVENT_TABLE()

CMoonPanel::CMoonPanel(wxWindow* parent, int id, const wxPoint& pos, const wxSize& size) :
wxPanel(parent, id, pos, size),
wxTimer(),
m_moon(),
m_bitmap(NULL),
m_width(0),
m_height(0)
{
	m_width  = size.GetWidth();
	m_height = size.GetHeight();

	m_bitmap = new wxBitmap(m_width, m_height);

	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);

	dc.BeginDrawing();
	dc.SetBackground(*wxBLACK_BRUSH);
	dc.Clear();
	dc.EndDrawing();

	Start(1000);
}


CMoonPanel::~CMoonPanel()
{
	delete m_bitmap;
}

void CMoonPanel::onPaint(const wxPaintEvent& event)
{
	wxPaintDC dc(this);

	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void CMoonPanel::Notify()
{
	wxDateTime now = wxDateTime::UNow();

	double dayNum = m_moon.getDayNum(now);
	m_moon.findMoon(dayNum);

	wxString az;
	az.Printf(wxT("Az: %3.1f"), m_moon.getAzimuth());

	wxString el;
	el.Printf(wxT("El: %2.1f"), m_moon.getElevation());

	wxString ra;
	ra.Printf(wxT("%3.2f"), m_moon.getRA());

	wxString dec;
	dec.Printf(wxT("%2.2f"), m_moon.getDec());

	wxString lha;
	lha.Printf(wxT("%3.2f"), m_moon.getLHA());

	wxString range;
	range.Printf(wxT("%3.0f"), m_moon.getRange() / 1000.0);

	wxString delay;
	delay.Printf(wxT("%.0f"), 2.0 * m_moon.getRange() / 299810.0);

	wxString doppler;
	double shift = m_moon.getDoppler();
	doppler.Printf(wxT("%c%3.1f"), (shift >= 0.0) ? wxT('+') : wxT('-'), ::fabs(shift));

	wxString echo;
	shift = 2.0 * m_moon.getDoppler();
	echo.Printf(wxT("%c%3.1f"), (shift >= 0.0) ? wxT('+') : wxT('-'), ::fabs(shift));

	wxString loss;
	double ploss = m_moon.getPathLoss();
	loss.Printf(wxT("%c%2.1f"), (ploss >= 0.0) ? wxT('+') : wxT('-'), ::fabs(ploss));

	wxMemoryDC mDC;
	mDC.SelectObject(*m_bitmap);
	mDC.BeginDrawing();

	mDC.SetBackground(*wxBLACK_BRUSH);
	mDC.Clear(),

	mDC.SetTextForeground(*wxWHITE);

	wxFont font2(50, wxDEFAULT, wxNORMAL, wxNORMAL);
	mDC.SetFont(font2);

	writeText(mDC, az, m_width / 2, 10, Text_Centred);
	writeText(mDC, el, m_width / 2, 70, Text_Centred);

	wxFont font3(14, wxDEFAULT, wxNORMAL, wxNORMAL);
	mDC.SetFont(font3);
	writeText(mDC, wxT("RA:"),    m_width / 2 - 170, m_height / 2 + 0,  Text_Left);
	writeText(mDC, ra,            m_width / 2 - 60,  m_height / 2 + 0,  Text_Right);
	writeText(mDC, wxT("deg"),    m_width / 2 - 55,  m_height / 2 + 0,  Text_Left);
	writeText(mDC, wxT("Dec:"),   m_width / 2 - 170, m_height / 2 + 20, Text_Left);
	writeText(mDC, dec,           m_width / 2 - 60,  m_height / 2 + 20, Text_Right);
	writeText(mDC, wxT("deg"),    m_width / 2 - 55,  m_height / 2 + 20, Text_Left);
	writeText(mDC, wxT("LHA:"),   m_width / 2 - 170, m_height / 2 + 40, Text_Left);
	writeText(mDC, lha,           m_width / 2 - 60,  m_height / 2 + 40, Text_Right);
	writeText(mDC, wxT("deg"),    m_width / 2 - 55,  m_height / 2 + 40, Text_Left);
	writeText(mDC, wxT("Range:"), m_width / 2 - 170, m_height / 2 + 60, Text_Left);
	writeText(mDC, range,         m_width / 2 - 60,  m_height / 2 + 60, Text_Right);
	writeText(mDC, wxT("kms"),    m_width / 2 - 55,  m_height / 2 + 60, Text_Left);
	writeText(mDC, wxT("Delay:"), m_width / 2 - 170, m_height / 2 + 80, Text_Left);
	writeText(mDC, delay,         m_width / 2 - 60,  m_height / 2 + 80, Text_Right);
	writeText(mDC, wxT("ms"),     m_width / 2 - 55,  m_height / 2 + 80, Text_Left);

	writeText(mDC, wxT("Doppler:"), m_width / 2 + 0,   m_height / 2 + 0,  Text_Left);
	writeText(mDC, doppler,         m_width / 2 + 130, m_height / 2 + 0,  Text_Right);
	writeText(mDC, wxT("Hz"),       m_width / 2 + 135, m_height / 2 + 0,  Text_Left);
	writeText(mDC, wxT("Echo:"),    m_width / 2 + 0,   m_height / 2 + 20, Text_Left);
	writeText(mDC, echo,            m_width / 2 + 130, m_height / 2 + 20, Text_Right);
	writeText(mDC, wxT("Hz"),       m_width / 2 + 135, m_height / 2 + 20, Text_Left);
	writeText(mDC, wxT("Path:"),    m_width / 2 + 0,   m_height / 2 + 40, Text_Left);
	writeText(mDC, loss,            m_width / 2 + 130, m_height / 2 + 40, Text_Right);
	writeText(mDC, wxT("dB"),       m_width / 2 + 135, m_height / 2 + 40, Text_Left);

	mDC.EndDrawing();
	mDC.SelectObject(wxNullBitmap);

	wxClientDC cDC(this);
	cDC.BeginDrawing();
	cDC.DrawBitmap(*m_bitmap, 0, 0, false);
	cDC.EndDrawing();
}

void CMoonPanel::writeText(wxDC& dc, const wxString& text, int x, int y, enum textPos pos) const
{
	wxCoord width, height;
	dc.GetTextExtent(text, &width, &height);

	switch (pos) {
		case Text_Left:
			break;
		case Text_Right:
			x -= width;
			break;
		case Text_Centred:
			x -= width / 2;
			break;
		default:
			wxASSERT(false);
			break;
	}

	wxASSERT(x >= 0 && x < m_width);
	wxASSERT(y >= 0 && y < m_height);

	dc.DrawText(text, x, y);
}

bool CMoonPanel::setLocator(const wxString& locator)
{
	CLocator loc(locator);

	if (!loc.isValid())
		return false;

	m_moon.setLocation(loc.getLatitude(), loc.getLongitude());

	return true;
}

void CMoonPanel::setFrequency(double frequency)
{
	wxASSERT(frequency > 0.0);

	m_moon.setFrequency(frequency);
}
