/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX/HB9DRD
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "common/Exception.h"
#include "common/NCO.h"

#include "JT44Lookups.h"
#include "JT44Send.h"
#include "JT44Defs.h"

#include "jt44gui/JT44App.h"

#include <wx/datetime.h>
#include <wx/debug.h>
#include <wx/log.h>

CJT44Send::CJT44Send(CSoundDev* soundDev, CPTTPort* pttPort, wxString* message, bool managed) :
CSend(managed),
m_soundDev(soundDev),
m_pttPort(pttPort),
m_message(message)
{
	wxASSERT(m_soundDev != NULL);
	wxASSERT(m_pttPort != NULL);
	wxASSERT(m_message != NULL);
}

CJT44Send::~CJT44Send()
{
}

void CJT44Send::run()
{
	::wxLogInfo(wxT("%ld: Starting JT44Send"), GetId());

	wxASSERT(m_soundDev != NULL);
	wxASSERT(m_pttPort != NULL);
	wxASSERT(m_message != NULL);

	m_pttPort->keyTX();

	waitForAudioStart();

	m_soundDev->openWrite();

	m_message->resize(JT44_MESSAGE_LENGTH, wxT(' '));

	CJT44Lookups lookups;
	CNCO nco(JT44_SAMPLE_RATE, JT44_SYMBOL_LENGTH);

	for (int i = 0; i < 135 && !TestDestroy(); i++) {
		int pos = lookups.lookupPosition(i);
		int tone;

		if (pos == -1) {
			tone = 118 * 2;
		} else {
			int n = lookups.lookupChar(m_message->GetChar(pos));

			if (n == -1)
				n = lookups.lookupChar(wxT(' '));

			tone = n * 2;
		}

		double freq = double(tone) * double(JT44_SAMPLE_RATE) / double(JT44_SYMBOL_LENGTH);

		double* out = nco.generate(freq);

		m_soundDev->write(out, JT44_SYMBOL_LENGTH);
	}

	m_soundDev->close();

	m_pttPort->unkeyTX();

	::wxLogInfo(wxT("%ld: JT44Send ending"), GetId());
}

void CJT44Send::waitForAudioStart() const
{
	wxDateTime now = wxDateTime::UNow();

	switch (now.GetSecond()) {
		case 1:
		case 31:
			return;
		case 0:
		case 30:
			Sleep(1000 - now.GetMillisecond());
			break;
		default:
			throw CException(wxT("Weirdness in JT44 send timer"));
	}
}

void CJT44Send::error(const wxString& text)
{
        ::wxLogError(wxT("%ld: %s"), GetId(), text.c_str());

	::wxGetApp().error(text);
}
