/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX/HB9DRD
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/time.h>
#include <math.h>

#include "common/FFT.h"
#include "common/SFFT.h"
#include "common/SoundFile.h"

#include "fsk441/FSK441Defs.h"
#include "fsk441/FSK441Lookups.h"

#include "FFTSpeed.h"

int main(int argc, char **argv)
{
	if (argc < 3) {
		::fprintf(stderr, "Usage: FFTSpeed <filename> <overlap>\n");
		return 1;
	}

	wxString fileName = wxString(argv[1]);
	int       overlap = ::atoi(argv[2]);

	CFFTSpeed speed(fileName, overlap);
	speed.run();

	return 0;
}

CFFTSpeed::CFFTSpeed(const wxString& fileName, int overlap) :
m_fileName(fileName),
m_overlap(overlap),
m_audioData(NULL),
m_count(0)
{
	m_audioData = new double[30 * FSK441_SAMPLE_RATE];
}

CFFTSpeed::~CFFTSpeed()
{
	delete[] m_audioData;
}

void CFFTSpeed::run()
{
	CSoundDev* file = new CSoundFile;

	file->openRead(m_fileName, FSK441_SAMPLE_RATE, 16);

	double* in = new double[4096];

	while (m_count < (30 * FSK441_SAMPLE_RATE)) {
		int len = 4096;
		if (!file->read(in, len))
			break;

		for (int i = 0; i < len; i++) {
			m_audioData[m_count] = in[i];
			m_count++;
		}
	}

	file->close();

	delete file;
	delete[] in;

	processFFT();

	processSFFT();
}

void CFFTSpeed::processSFFT() const
{
	struct timeval tvBegin;
	::gettimeofday(&tvBegin, NULL);

	CSFFT sfft(FSK441_FFT_LENGTH, FSK441_BIN0, FSK441_BIN3 + 1);

	for (int i = 0; i < (m_count - FSK441_FFT_LENGTH); i++)
		double* bins = sfft.process(m_audioData[i]);

	struct timeval tvEnd;
	::gettimeofday(&tvEnd, NULL);

//	::printf("SFFT: start at %ld:%ld end at %ld:%ld\n", tvBegin.tv_sec, tvBegin.tv_usec, tvEnd.tv_sec, tvEnd.tv_usec);

	::printf("SFFT: elapsed: %ld seconds\n", tvEnd.tv_sec - tvBegin.tv_sec);
}

void CFFTSpeed::processFFT() const
{
	struct timeval tvBegin;
	::gettimeofday(&tvBegin, NULL);

	CFFT fft(FSK441_FFT_LENGTH);

	int done = 0;

	for (int i = 0; i < (m_count - FSK441_FFT_LENGTH); i += m_overlap) {
		done++;
		double* bins = fft.process(m_audioData + i);
	}

	struct timeval tvEnd;
	::gettimeofday(&tvEnd, NULL);

//	::printf("FFT: start at %ld:%ld end at %ld:%ld\n", tvBegin.tv_sec, tvBegin.tv_usec, tvEnd.tv_sec, tvEnd.tv_usec);

	::printf("FFT: elapsed: %ld seconds\n", tvEnd.tv_sec - tvBegin.tv_sec);
	::printf("FFT: performed %d FFTs\n", done);
}
